AuthorisationContext = (function () {

    var Observable = function (def) {
        this.callbacks = [];
        this.data = undefined;

        if (def != undefined) {
            this.data = def;
        }
    }
    Observable.prototype.deliver = function (data) {
        this.data = data;
        for (var i in this.callbacks) {
            this.callbacks[i](data);
        }
        return this;
    }
    Observable.prototype.subscribe = function (callback) {
        this.callbacks.push(callback);
        if (this.data != undefined) {
            callback(this.data);
        }
        return this;
    }

    AuthorisationContext = function (config) {
        this.roles = null;
        this.privileges = null;
        this.promise = {};
        this.getDataObservable = new Observable();
        this.data = {};
        this.config = config;

        this.context = new AuthenticationContext(config);
        this.context.handleWindowCallback();
        this.currentOrganizationId = null;
        if (this.config.organizationId) {
            this.currentOrganizationId = this.config.organizationId;
            this.getData();
        }
    };

    AuthorisationContext.prototype.context = function () {
        this.context;
    }
    AuthorisationContext.prototype.login = function () {
        this.context.login();
    }
    AuthorisationContext.prototype.logout = function () {
        this.context.logOut();
    }
    AuthorisationContext.prototype.handleCallback = function () {
        this.context.handleCallback();
    }
    AuthorisationContext.prototype.login = function () {
        this.context.login();
    }

    AuthorisationContext.prototype.hasRole = function (role) {
        var result = new Observable(false);
        this.getRoles().subscribe(roles => result.deliver(roles ? roles.some(r => r === role) : false));
        return result;
    }
    AuthorisationContext.prototype.hasPrivilege = function (privilege) {
        var result = new Observable(false);
        this.getPrivileges().subscribe(privileges => result.deliver(privileges ? privileges.some(p => p === privilege) : false));
        return result;
    }
    AuthorisationContext.prototype.getRoles = function () {
        var result = new Observable(null);
        this.getData().subscribe(data => {
            result.deliver(data ? data.map(r => r.RoleTag) : null)
        });
        return result;F
    }
    AuthorisationContext.prototype.getPrivileges = function () {
        var result = new Observable(null);
        this.getData().subscribe(data => {
            result.deliver(data ? data.reduce((acc, value) => { return acc.concat(value.Privileges.map(p => p.PrivilegeTag)) }, []) : null);
        });
        return result;
    }

    AuthorisationContext.prototype.userInfo = function () {
        if (this.context) {
            return this.context.getCachedUser();
        }
        return {};
    }
    AuthorisationContext.prototype.isAuthenticated = function () {
        return this.userInfo() && this.context.getCachedToken(this.config.clientId);
    }
    AuthorisationContext.prototype.getOrganizationId = function () {
        return this.currentOrganizationId;
    }
    AuthorisationContext.prototype.setOrganizationId = function (value) {
        this.currentOrganizationId = value;
        //notify?
        this.getData();
    }

    AuthorisationContext.prototype.getData = function () {
        var orgId = this.currentOrganizationId;
        if (!orgId) {
            return this.getDataObservable.deliver(null);
        }

        if (!this.isAuthenticated()) {
            this.context.login();
        }

        if (this.data[orgId]) {
            return this.getDataObservable.deliver(this.data[orgId]);
        }
        if (this.promise[orgId]) {
            this.promise[orgId].then(data => this.getDataObservable.deliver(data));
            return this.getDataObservable;
        }

        let that = this;

        this.promise[orgId] = new Promise(function (resolve, reject) {

            that.context.acquireToken(that.config.getRoleResource, (err, token) => {

                fetch(that.config.getRoleUrl + orgId, {
                    headers: {
                        'Ocp-Apim-Subscription-Key': that.config.ocpApimSubscriptionKey,
                        'OrganizationId': orgId,
                        'Accept': '*/*',
                        'Authorization': 'Bearer ' + token
                    },
                    method: 'GET',
                    mode: 'cors',
                })
                    .then(response => response.json())
                    .then(data => {
                        that.data[orgId] = data.Result;
                        resolve(data.Result);
                    });
            });
        });
        this.promise[orgId].then(data => this.getDataObservable.deliver(data));
        return this.getDataObservable;
    }

    return AuthorisationContext;
}());
